import sys
import pygame
from pygame.locals import *
from math import pi, sin, cos
from random import randint, choice

import gp2x

def debug(text):
	#print text
	pass

pygame.init()
screen = pygame.display.set_mode((320,240))


# Handle the main game board
class board:
	def __init__(self, center_x, center_y, pairs):
		self.center_x = center_x
		self.center_y = center_y
		self.pairs = pairs
		self.pieces = []
		# For the background image
		self.background = pygame.image.load("data/background.png").convert()
		screen.blit(self.background, self.background.get_rect())
		# List containing all avaiable tools
		self.tools = [self.spin, self.swap]
		self.itool = 0 # Tool counter
		self.tool = self.tools[0] # Function pointer
		self.toolimg = []
		self.toolimg.append(pygame.image.load("data/tool1.png").convert_alpha())
		self.toolimg.append(pygame.image.load("data/tool2.png").convert_alpha())
		self.toolrect = self.toolimg[self.itool].get_rect()
		self.tool_x = 230
		self.tool_y = 120
		self.toolrect.left = self.tool_x
		self.toolrect.top = self.tool_y
		screen.blit(self.toolimg[0], self.toolrect)

		# Load all buttons ant put them in the list
		for i in range(pairs):
			filename = "data/button" + str(i + 1) + ".png"
			self.pieces.append(piece(pygame.image.load(filename).convert_alpha(), i, pairs, center_x, center_y))
		for i in range(pairs):
			filename = "data/button" + str(i + 1) + ".png"
			self.pieces.append(piece(pygame.image.load(filename).convert_alpha(), i + pairs, pairs, center_x, center_y))
	
		self.random_level(50)

	# If direction is -1 then rotate right, and if 1 then rotate left
	def rotate(self, direction):
		if direction < 0:



			for i in range(self.pairs * 2):
				self.pieces[self.find_by_pos(i)].pos -= 1
			self.pieces[self.find_by_pos(self.pairs - 1)].pos = self.pairs * 2 - 1
			self.pieces[self.find_by_pos(-1)].pos = self.pairs - 1
			debug("Rotating -1")





		else:
			i = self.pairs * 2 - 1
			while i >= 0:
				self.pieces[self.find_by_pos(i)].pos += 1
				i -= 1
			self.pieces[self.find_by_pos(self.pairs)].pos = 0
			self.pieces[self.find_by_pos(self.pairs * 2)].pos = self.pairs
			debug("Rotating 1")
		for p in self.pieces:
			p.moving = 1


	
	# Rotate the inner ring only
	def spin(self, direction):
		if direction < 0:
			for i in range(self.pairs):
				self.pieces[self.find_by_pos(i)].pos -= 1
			self.pieces[self.find_by_pos(-1)].pos = self.pairs - 1
			debug("Spinning -1")
		else:
			i = self.pairs - 2
			self.pieces[self.find_by_pos(self.pairs - 1)].pos = -1
			while i >= 0:
				self.pieces[self.find_by_pos(i)].pos += 1
				i -= 1
			self.pieces[self.find_by_pos(-1)].pos = 0
			debug("Spinning 1")
		for i in range(self.pairs):
			self.pieces[self.find_by_pos(i)].moving = 1
		
	
	# Swap the two upmoast located pieces
	def swap(self, direction):
		temp1 = self.pieces[self.find_by_pos(0)]
		temp2 = self.pieces[self.find_by_pos(self.pairs)]
		temp1.pos = self.pairs
		temp2.pos = 0
		debug("Swapping")

	
	# Find the piece with given position and return its index
	def find_by_pos(self, pos):
		for i in range(self.pairs * 2):
			if self.pieces[i].pos == pos: return i
	
	def next_tool(self):
		self.itool += 1
		if self.itool == len(self.tools):
			self.itool = 0
		self.tool = self.tools[self.itool]
		self.toolrect = self.toolimg[self.itool].get_rect()
		self.toolrect.left = self.tool_x
		self.toolrect.top = self.tool_y
	
	# Return 1 if board is clear, 0 otherwize
	def is_clear(self):
		clear = 1
		for i in range(self.pairs):
			if self.pieces[self.find_by_pos(i)].color != self.pieces[self.find_by_pos(i + self.pairs)].color:
				clear = 0
		return clear
	
	# Randomize a level (iters numer of operations)
	def random_level(self, iters):
		for i in range(iters):
			"""
			# First, choose a tool and use a number of times
			tool = choice(self.tools)
			for j in range(randint(0, self.pairs) * 2):
				tool(choice([1, -1]))
			"""
			# Spin some times
			for j in range(randint(0, self.pairs)):
				self.spin(1)

			# Swap once
			self.swap(1)

			# Rotate the wheel a number of times
			for j in range(randint(0, self.pairs)):
				self.rotate(1)
		if self.is_clear():
			self.random_level(iters)

		
# This class handles the game pieces
class piece:
	# Pos is a position number.
	def __init__(self, bitmap, pos, pairs, center_x, center_y):
		self.bitmap = bitmap
		self.pos = pos
		self.rect = self.bitmap.get_rect()
		# Angle between the buttons
		self.to_Angle = pos * 2 * pi / pairs
		self.current_Angle = pos * 2 * pi / pairs
		self.moving = 0
		if pos < pairs:
			self.color = pos
		else:
			self.color = pos - pairs
		self.position(pairs, center_x, center_y)
	
	def position(self, pairs, center_x, center_y):
		# Angle between the buttons
		Angle = 2 * pi / pairs

		# Distance from the global centrum to the centrum of this button
		distance = self.rect.width / (2 * sin(Angle / 2)) / 1.1 # 1.1

		if self.pos < pairs:
			self.to_Angle = self.pos * Angle
		else:
			self.to_Angle = (self.pos - pairs) * Angle

		self.velocity = 0.15  # 0.10
		"""
		if self.to_Angle < self.current_Angle:
			self.velocity += self.current_Angle - self.to_Angle
		if self.to_Angle > self.current_Angle:
			self.velocity -= self.to_Angle - self.current_Angle
		"""

		if int(self.current_Angle * 2 * pi) / (2 * pi) == int(self.to_Angle * 2 * pi) / (2 * pi):
			self.moving = 0

		if self.moving:
			# Find out which direction to go
			#if int(self.current_Angle / Angle) < int(self.to_Angle / Angle):
			if self.current_Angle < self.to_Angle:
				if self.to_Angle - self.current_Angle < pi:
					self.current_Angle += self.velocity
				else:
					self.current_Angle -= self.velocity
			#elif int(self.current_Angle / Angle) > int(self.to_Angle / Angle):
			elif self.current_Angle > self.to_Angle:
				if self.current_Angle - self.to_Angle < pi:
					self.current_Angle -= self.velocity
				else:
					self.current_Angle += self.velocity
			if self.current_Angle > 2 * pi:
				self.current_Angle -= 2 * pi
			if self.current_Angle < 0:
				self.current_Angle += 2 * pi

		# Inner ring
		if self.pos < pairs:
			#self.rect.left = center_x + distance * sin(self.current_Angle)
			#self.rect.top = center_y + distance * cos(self.current_Angle)
			self.rect.left = center_x + 4 + distance * sin(self.current_Angle)
			self.rect.top = center_y + 2 + distance * cos(self.current_Angle)
		# Outer ring
		else:
			#self.rect.left = center_x + (distance + self.rect.width ) * sin(self.current_Angle)
			#self.rect.top = center_y + (distance + self.rect.width ) * cos(self.current_Angle)
			self.rect.left = center_x + (distance + self.rect.width / 1.5) * sin(self.current_Angle)
			self.rect.top = center_y + (distance + self.rect.width / 3) * cos(self.current_Angle)

		screen.blit(self.bitmap, self.rect)

def controller_tick(gameboard):
	for event in pygame.event.get():
		if event.type == pygame.KEYDOWN:
			if event.key == pygame.K_LEFT: 
				gameboard.rotate(-1)
			if event.key == pygame.K_RIGHT:
				gameboard.rotate(1)
			if event.key == pygame.K_UP:
				gameboard.tool(1)
			if event.key == pygame.K_DOWN:
				gameboard.tool(-1)
			if event.key == pygame.K_q: return 0 
			if event.key == pygame.K_LALT:
				gameboard.next_tool()
		if event.type == pygame.JOYBUTTONDOWN:
			if event.button == gp2x.BUTTON_LEFT or event.button == gp2x.BUTTON_UPLEFT or event.button == gp2x.BUTTON_DOWNLEFT or event.button == gp2x.BUTTON_L: 
				gameboard.rotate(-1)
			if event.button == gp2x.BUTTON_RIGHT or event.button == gp2x.BUTTON_UPRIGHT or event.button == gp2x.BUTTON_DOWNRIGHT or event.button == gp2x.BUTTON_R: 
				gameboard.rotate(1)
			if event.button == gp2x.BUTTON_B:
				gameboard.tool(1)
			if event.button == gp2x.BUTTON_X:
				gameboard.tool(-1)
			if event.button == gp2x.BUTTON_START: 
				return 0 
			if event.button == gp2x.BUTTON_A or event.button == gp2x.BUTTON_Y:
				gameboard.next_tool()

		if event.type == pygame.QUIT:
			return 0
	if gameboard.is_clear():
		return 2
	return 1

def view_tick(gameboard):
	#screen.fill((0, 0, 0))
	screen.blit(gameboard.background, gameboard.background.get_rect())
	for p in gameboard.pieces:
		p.position(gameboard.pairs, gameboard.center_x, gameboard.center_y)
	screen.blit(gameboard.toolimg[gameboard.itool], gameboard.toolrect)
	pygame.display.flip()

def main():
	pairs = 3
	pos_x = 120
	pos_y = 100
	
    	pygame.mouse.set_visible(0)

    	num_joysticks = pygame.joystick.get_count()
    	if num_joysticks > 0:
        	stick = pygame.joystick.Joystick(0)
        	stick.init() # now we will receive events for the joystick

	gameboard = board(pos_x, pos_y, pairs)
	pygame.display.flip()

	while 1:
		#event = pygame.event.wait()
		while 1:
			result = controller_tick(gameboard)
			if result == 0: return 0
			if result == 2:
				for i in range(10):
					for p in gameboard.pieces:
						p.position(gameboard.pairs, gameboard.center_x, gameboard.center_y)
					view_tick(gameboard)
				pairs += 1
				if pairs == 8:
					print "You beat the game! Replay at first level."
					pairs = 3
				gameboard = board(pos_x, pos_y, pairs)
			
			# Redraw them
			view_tick(gameboard)

if __name__ == "__main__":
	main()

